import numpy as np
from ngsolve import *
from netgen.csg import *

#creating the domain
def MakeGeometry():
    geometry = CSGeometry()
    #Create the Individual Components
    Domain = OrthoBrick(Pnt(-100,-100,-100),Pnt(100,100,100)).bc("outer")
    MySphere = Sphere (Pnt(0,0,0), 1).maxh(0.1).mat("object")
    
    #Combine the Geometries for the objects you're creating
    geometry.Add ((Domain-MySphere).mat("air"))
    geometry.Add (MySphere)
    return geometry
    
ngmesh = MakeGeometry().GenerateMesh(maxh=15)

#Creating the mesh and defining the element types
mesh = Mesh(ngmesh)
mesh.Curve(5)
fes = HCurl(mesh, order=3, dirichlet="outer", flags = { "nograds" : True }, complex=True)

#Setting up coefficients
epsi=10**-8
inorout = {"object" : 1, "air" :0}
inout_coef = [inorout[mat] for mat in mesh.GetMaterials() ]
inout = CoefficientFunction(inout_coef)

#Set up the boundary and grid function
Bound=CoefficientFunction( (0,-z,y) )# u= e_1 x X
ComplexBound=Bound+(2*Bound*1j)
Theta=GridFunction(fes)
Theta.Set(ComplexBound, BND)

#Set up the bilinear form
u = fes.TrialFunction()
v = fes.TestFunction()
f = LinearForm(fes)
a = BilinearForm(fes)
a += SymbolicBFI( InnerProduct( curl(u) , curl(v) ))
a += SymbolicBFI( epsi * (u * v) )

#Precondition and solve
c = Preconditioner(a, "bddc")
a.Assemble()
f.Assemble()
c.Update()
res = f.vec.CreateVector()
res.data = f.vec - a.mat * Theta.vec
inverse= GMRESSolver(a.mat, c.mat, precision=10**-6, maxsteps=200)
Theta.vec.data += inverse * res

Exact=Integrate(inout * InnerProduct(ComplexBound,ComplexBound),mesh)
Calculated=Integrate(inout * InnerProduct(Theta,Theta),mesh)
HalfandHalf=Integrate(inout * InnerProduct(Theta,ComplexBound),mesh)

print("Exact=", Exact)
print("Calculated=", Calculated)
print("HalfandHalf=", HalfandHalf)